/* Copyright 2014 Marcus Wichelmann
*
* This file is part of evnc.
*
* evnc is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* evnc is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with evnc. If not, see http://www.gnu.org/licenses/.
*/

namespace evnc.Widgets {
	public class VncConnection : Granite.Widgets.Tab {
		public signal void on_save_screenshot_exception (string message);

		public string host;
		public string port;
		public string user;
		public string desc;

		private SettingsManager options;

		private Gtk.Grid grid;

		private Gtk.ScrolledWindow scrolledwindow;

		private Gtk.Viewport viewport;

		public Vnc.Display display;

		public VncConnection (string host, string port, string user) {
			desc = _("Connecting...");
			this.label = desc;

			options = new SettingsManager ();
			options.on_display_mode_changed.connect ((val) => {
				set_size_mode (val);
			});

			grid = new Gtk.Grid ();
			scrolledwindow = new Gtk.ScrolledWindow (null, null);
			viewport = new Gtk.Viewport (null, null);
			display = new Vnc.Display ();

			viewport.add (display);
			scrolledwindow.add (viewport);
			grid.attach (scrolledwindow, 0, 0, 1, 1);

			scrolledwindow.set_hexpand (true);
			scrolledwindow.set_vexpand (true);
			scrolledwindow.draw.connect (() => {
				if (options.display_mode == 0) {
					update_view_border ();
				}

				return false;
			});

			var bg_color = Gdk.RGBA();
			bg_color.red = 0;
			bg_color.green = 0;
			bg_color.blue = 0;
			bg_color.alpha = 1;
			scrolledwindow.override_background_color (Gtk.StateFlags.NORMAL, bg_color);

			this.page = grid;

			display.set_credential (Vnc.DisplayCredential.USERNAME, user);
			display.set_credential (Vnc.DisplayCredential.CLIENTNAME, "evnc");
			display.set_pointer_grab (true);
			display.set_keyboard_grab (true);
			display.set_pointer_local (options.pointer_local);
			display.set_lossy_encoding (options.lossy_encoding);
			display.vnc_disconnected.connect (() => {
				this.close ();
			});
			display.vnc_connected.connect (() => {
				desc = _("Logging in...");
				this.label = desc;
			});
			display.vnc_initialized.connect (() => {
				desc = "";
				if (user != "") {
					desc += user + "@";
				}
				desc += host + ":" + port;
				this.label = desc;
			});
			display.vnc_auth_credential.connect ((cred) => {
				var input_box = new Gtk.MessageDialog (null, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
						Gtk.MessageType.WARNING, Gtk.ButtonsType.OK_CANCEL, _("Password required!"));
				input_box.title = _("Connect to:") + " " + host;

				var pass_entry = new Gtk.Entry ();
				pass_entry.set_size_request (400, 0);
				pass_entry.set_visibility (false);
				pass_entry.caps_lock_warning = true;
				pass_entry.set_placeholder_text (_("Password"));
				pass_entry.margin_start = 12;
				pass_entry.margin_end = 12;

				input_box.get_content_area ().pack_end (pass_entry, true, false, 0);
				input_box.show_all ();

				if (input_box.run () == Gtk.ResponseType.OK) {
					display.set_credential (Vnc.DisplayCredential.PASSWORD, pass_entry.get_text ());
				} else {
					this.close ();
				}

				input_box.destroy ();
			});

			set_color_depth (options.color_depth);
			options.settings.bind("readonly", display, "read_only", SettingsBindFlags.DEFAULT);

			if (display.open_host (host, port)) {
				this.host = host;
				this.port = port;
				this.user = user;
			}

			set_size_mode  (options.display_mode);
		}

		public void take_screenshot () {
			var img = display.get_pixbuf ();

			var save_dialog = new Gtk.FileChooserDialog (_("Save the screenshot"), null, Gtk.FileChooserAction.SAVE,
					_("Cancel"), Gtk.ResponseType.CANCEL, _("Save"), Gtk.ResponseType.ACCEPT);

			var file_filter = new Gtk.FileFilter ();
			file_filter.add_mime_type ("image/jpeg");

			save_dialog.set_filter (file_filter);

			var preview_image = new Gtk.Image ();
			preview_image.set_from_pixbuf (img.scale_simple (150, (int)(((float)150 / (float)img.width) * img.height), Gdk.InterpType.BILINEAR));

			save_dialog.set_preview_widget (preview_image);

			preview_image.show ();

			if (save_dialog.run () == Gtk.ResponseType.ACCEPT) {
				try {
					img.save (save_dialog.get_filename (), "jpeg");
				} catch (Error ex) {
					on_save_screenshot_exception (ex.message);
				}
			}

			save_dialog.close ();
			save_dialog.destroy ();
		}

		private void update_view_border () {
			if (scrolledwindow.get_allocated_width () > 0 && scrolledwindow.get_allocated_height () > 0 && display.width > 0 && display.height > 0) {
				if (((float)scrolledwindow.get_allocated_width () / (float)display.width) * (float)display.height > scrolledwindow.get_allocated_height ()) {
					int border_size = (int)(((float)scrolledwindow.get_allocated_width () - (((float)scrolledwindow.get_allocated_height () / (float)display.height) * (float)display.width)) / 2);
					viewport.margin_start = border_size;
					viewport.margin_end = border_size;
					viewport.margin_top = 0;
					viewport.margin_bottom = 0;
				} else {
					int border_size = (int)(((float)scrolledwindow.get_allocated_height () - (((float)scrolledwindow.get_allocated_width () / (float)display.width) * (float)display.height)) / 2);
					viewport.margin_start = 0;
					viewport.margin_end = 0;
					viewport.margin_top = border_size;
					viewport.margin_bottom = border_size;
				}
			}
		}

		private void set_size_mode (int val) {
			if (val == 0) {
					display.force_size = false;
					display.scaling = true;
					display.set_size_request (0, 0);

					update_view_border ();
				} else {
					viewport.margin = 0;

					display.force_size = true;
					display.scaling = false;
					display.set_size_request (display.width, display.height);
				}
		}

		private void set_color_depth (string val) {
			switch (val) {
				case "default":
					display.set_depth (Vnc.DisplayDepthColor.DEFAULT);
					break;
				case "full":
					display.set_depth (Vnc.DisplayDepthColor.FULL);
					break;
				case "medium":
					display.set_depth (Vnc.DisplayDepthColor.MEDIUM);
					break;
				case "low":
					display.set_depth (Vnc.DisplayDepthColor.LOW);
					break;
				case "ultra low":
					display.set_depth (Vnc.DisplayDepthColor.ULTRA_LOW);
					break;
			}
		}
	}
}